#include "TrivialDetector.hh"
#include "G4LogicalVolume.hh"
#include "G4VPhysicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4Material.hh"
#include "G4NistManager.hh"
#include "G4Box.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "globals.hh"

// The user-configurable settings
#include "TrivialSimSettings.hh"

TrivialDetector::TrivialDetector()
{
  G4cout << "-- TrivialDetector constructed." << G4endl;
}

TrivialDetector::~TrivialDetector()
{
  G4cout << "-- TrivialDetector deleted." << G4endl;
}


/**
 * This overloaded method is the entry point for defining the
 * simulation setup: geometry, materials, and sensitive detectors.
 * In this trivial case, our detector consists simply of a small
 * target cube.This target is placed into the world volume of the
 * simulation - another, bigger cube.The material of the target
 * is lead (Pb) per default and can be changed via the command-line. 
 *
 * The size of the target cube should be parameterizable via the
 * command line using a command defined in this exercise.
 * <b>EXERCISE: you have to implement this!</b>
 */
G4VPhysicalVolume* TrivialDetector::Construct() 
{

  G4cout << "-- TrivialDetetor::Construct() entered." << G4endl;

  // Get hold of the user-configurable settings
  const TrivialSimSettings * settings = TrivialSimSettings::getSettings();

  G4String worldName("World");

  // Here we define a simple box of 2m length on each side.
  // This will be the solid of our world volume.
  G4VSolid * worldSolid = new G4Box(worldName, 1*m, 1.*m, 1.*m);

  // The material of the world volume should be simply Air.
  // We could construct air out of its elements, or we can just
  // get it from the G4 built-in database of materials. We will
  // demonstrate the latter option:
  
  // Fetch the singleton manager of the built-in material database.
  G4NistManager* matManager = G4NistManager::Instance();

  // Get the material by name from the built-in material database:
  G4Material * worldMaterial = matManager->FindOrBuildMaterial("G4_AIR");

  G4LogicalVolume * worldLV = 
    new G4LogicalVolume(worldSolid, //< solid of the logical volume
			worldMaterial, //< material of the logical volume
			worldName); //< null pointer to indicate that 
                                    //  it's not a sensitive detector

  // The following defines the color of the world for visualization purposes:
  worldLV->SetVisAttributes(new G4VisAttributes(G4Colour::White()));
  
  // Into the world logical-volume (worldLV) we will now place 
  // a small target cube made of a configurable material:
  G4String targetName("Target");

  // Here we fetch the material from our settings-singleton.
  G4Material * targetMaterial = settings->getTargetMaterial();
  G4cout << "-- Target material is " << targetMaterial->GetName() << G4endl;

  ////////////////////////////////////////////////////////////////////
  //
  // Definition of a the target solid:
  //
  // The target solid is a cube. The length of a size of this cube
  // is a user-configurable parameter. The value of the parameter is 
  // held in the settings singleton (there it is modified by the
  // value given by the G4 command line.
  // 
  // Look at the code of the TrivialSimSettings singleton. 
  // Get hold of the singleton and fetch the parameter which
  // specifies the size of the target in order to define
  // the targets solid below:
  //
  
  G4double sz = TrivialSimSettings::getSettings()->getTargetSize();
  G4VSolid * targetSolid = new G4Box(targetName, sz/2., sz/2., sz/2.);
  G4cout << "-- Target built with size " << sz/cm << " cm" << G4endl;

  // 
  // Now the solid has to be associated with its material by
  // means of a G4LogicalVolume - construct it below:
  //
  
  G4LogicalVolume * targetLV = 
    new G4LogicalVolume(targetSolid, targetMaterial, targetName);

  //
  //////////////////////////////////////////////////////////////////////


  // The following defines the color of the target for visualization purposes:
  targetLV->SetVisAttributes(new G4VisAttributes(G4Colour::Yellow()));

  // Here we place the target 0.5 meters off the center
  // along the x-axis within the world volume: */
  new G4PVPlacement(0, //< null pointer for an unrotated placement
		    G4ThreeVector(0.5*m, 0., 0.), //< put the target 0.5m along the x-axis
		    targetLV, //< the logical volume to be places = the target cube
		    targetName, //< a name for this placement
		    worldLV, //< the mother volume in which we place the target
		    false, //< ... not used
		    0); //< a copy number for the placement
  

  // Return the world volume of the simulation. This is simply an unrotated, 
  // not translated physical-volume, which is NOT placed into a mother 
  // logical-volume. Instead of a pointer to the mother logical-volume 
  // a null-pointer is provided:
  return new G4PVPlacement(0,//< unrotated
			   G4ThreeVector(0., 0., 0.), //< untranslated
			   worldLV, //< logical volume of the world volume
			   worldName, //< name of the world volume
			   0, //< no pointer to a mother volume
			   false, //< no boolean operation
			   0); //< copy number of the world volume
}

