#ifndef TrivialSimSettings_csc
#define TrivialSimSettings_csc

#include "G4Types.hh"

class G4Material;
class TrivialCommandLine;

/**
 * Singleton to hold the user-adjustable  configuration for the simulation.
 * This Singleton is used to manage all parameters relevant for 
 * our simulation. In this trivial case, it will only hold the material
 * that is used in the defintion of the target cube.
 *
 * Other examples could be:
 * 
 *   - dimensions for the detector description
 *   - material assignments for other detector parts
 *   - settings for the primary generator
 *   - etc. ...
 *
 * Example of usage:
 * \code
 * // Get the pointer to the singleton settings
 * TrivialSimSettings * settings = TrivialSimSettings::getSettings();
 *
 * // Access the settings to get the material for the target cube
 * G4Material* myMaterial = settings->getTargetMaterial();
 * \endcode
 */
class TrivialSimSettings 
{

  friend class TrivialCommandLine; /** CommandLine is allowed to change internals */

public:

  /** Get a singleton pointer to default simulation settings. */
  static const TrivialSimSettings * const getSettings(); 

  G4Material * getTargetMaterial() const; /**< Material of the target cube. */

private:

  TrivialSimSettings();  /**< Constructor defining default values for all settings. */
  ~TrivialSimSettings() {}; /**< Undesctructable **/

  /// Write acces to settings for friend classes only 
  static TrivialSimSettings * const getModifiableSettings();

  static TrivialSimSettings * instance_; /**< Pointer holding the singleton instance */

  // ------------------------------------------------------------------------
  // Detector description parameters 
  // ------------------------------------------------------------------------

  G4Material* targetMaterial_; /**< Material to be used in the defintion of the target cube. */

};

#endif
