#include "TrivialCommandLine.hh"

#include "TrivialSimSettings.hh"

#include "G4NistManager.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4Material.hh"

#include "Randomize.hh"

/**
 * Creates several G4 commands. A G4 command implements the command line
 * interface of G4. Commands are hierarchically organised in folders /
 * command directories. This class puts all commands into the "/csc"-
 * directory. 
 */
TrivialCommandLine::TrivialCommandLine()
{
  // Create a G4-command-line directory
  cmdDir_ = new G4UIdirectory("/csc/");
  cmdDir_->SetGuidance("special UI commands for CSC06");
  
  // A command for changing the material of the target cube
  targetMaterialCmd_ = new G4UIcmdWithAString("/csc/setTargetMaterial",this);
  targetMaterialCmd_->SetGuidance("Specify a material for the target.");
  targetMaterialCmd_->SetParameterName("materialName",false);
  targetMaterialCmd_->AvailableForStates(G4State_PreInit);

  G4cout << "-- TrivialCommandLine constructed. New commands are available."
	 << G4endl;
}


TrivialCommandLine::~TrivialCommandLine() 
{
  G4cout << "-- TrivialCommandLine destructed. " << G4endl;
}


/**
 * This method is overloaded from the base class. G4 calls this method
 * whenever a command is either entered interactively or processed 
 * from a G4 macro. Our class needs to find out, if the given command
 * belongs to it (i.e. if the pointer of the given command matches
 * a pointer of the commands created by TrivialCommandLine).
 * If this is the case, the values in the settings-singleton are
 * changed accordingly.
 * The method itself is called by G4 when it handles command-line
 * or macro input.
 * @param command instance of a G4 command, 
 * @param newValue value to be applied by the command
 */
void TrivialCommandLine::SetNewValue(G4UIcommand* command, G4String newValue)
{
  // Get hold of our configuration singleton.
  TrivialSimSettings * const settings = TrivialSimSettings::getModifiableSettings();
  
  // Handle a new material assignment for the target cube:
  if (command==this->targetMaterialCmd_) {
    
    // Save the pointer to the current material of the target:
    G4Material* oldMaterial = settings->targetMaterial_;

    // Try to get the material from the internal "database" of materials:
    G4Material *  newMaterial = G4NistManager::Instance()->FindOrBuildMaterial(newValue);

    // If the new material does not exists in the internal database,
    // issue an error message. No material will be changed for the target.
    if  ( ! newMaterial ) {
      G4cout << "Material is not defined!" << G4endl 
	     << "Not changing target material from " << oldMaterial->GetName()
	     << " to " << newValue << G4endl;
    }

    // If the new material could be retrieved from the internal database,
    // update the settings-singleton.
    else {
      settings->targetMaterial_ = newMaterial;
      G4cout << "Changed target material from " 
	     << oldMaterial->GetName() << " to " << newValue << G4endl;
      G4cout << "New material will be effected only when set in PreInit state!" 
	     << G4endl;
      
      // NOTE:
      // This command only updates the value in the settings singleton, it does
      // not change the material in the geometry! 
      // When the geometry is initialized through the user initialization class
      // TrivailDetector, this class will read the material value from the
      // singleton and use it for the definition of the target cube!
    }
  }


}
